@@ -2,6 +2,7 @@
  * The MIT License (MIT)
  * 
  * Copyright (c) 2015 Charles J. Cliffe
+ * Copyright (c) 2015-2017 Josh Blum
 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -197,6 +198,22 @@ bool SoapyRTLSDR::hasDCOffsetMode(const int direction, const size_t channel) con
     return false;
 }
 
+bool SoapyRTLSDR::hasFrequencyCorrection(const int direction, const size_t channel) const
+{
+    return true;
+}
+
+void SoapyRTLSDR::setFrequencyCorrection(const int direction, const size_t channel, const double value)
+{
+    ppm = int(value);
+    rtlsdr_set_freq_correction(dev, ppm);
+}
+
+double SoapyRTLSDR::getFrequencyCorrection(const int direction, const size_t channel) const
+{
+    return double(ppm);
+}
+
 /*******************************************************************
  * Gain API
  ******************************************************************/
