@@ -2,6 +2,7 @@
  * The MIT License (MIT)
  * 
  * Copyright (c) 2015 Charles J. Cliffe
+ * Copyright (c) 2015-2017 Josh Blum
 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,6 +25,7 @@
 
 #include "SoapyRTLSDR.hpp"
 #include <SoapySDR/Logger.hpp>
+#include <SoapySDR/Formats.hpp>
 #include <algorithm> //min
 #include <climits> //SHRT_MAX
 #include <cstring> // memcpy
@@ -32,9 +34,9 @@
 std::vector<std::string> SoapyRTLSDR::getStreamFormats(const int direction, const size_t channel) const {
     std::vector<std::string> formats;
 
-    formats.push_back("CS8");
-    formats.push_back("CS16");
-    formats.push_back("CF32");
+    formats.push_back(SOAPY_SDR_CS8);
+    formats.push_back(SOAPY_SDR_CS16);
+    formats.push_back(SOAPY_SDR_CF32);
 
     return formats;
 }
@@ -46,7 +48,7 @@ std::string SoapyRTLSDR::getNativeStreamFormat(const int direction, const size_t
      }
 
      fullScale = 128;
-     return "CS8";
+     return SOAPY_SDR_CS8;
 }
 
 SoapySDR::ArgInfoList SoapyRTLSDR::getStreamArgsInfo(const int direction, const size_t channel) const {
@@ -59,7 +61,7 @@ SoapySDR::ArgInfoList SoapyRTLSDR::getStreamArgsInfo(const int direction, const
 
     SoapySDR::ArgInfo bufflenArg;
     bufflenArg.key = "bufflen";
-    bufflenArg.value = "16384";
+    bufflenArg.value = std::to_string(DEFAULT_BUFFER_LENGTH);
     bufflenArg.name = "Buffer Size";
     bufflenArg.description = "Number of bytes per buffer, multiples of 512 only.";
     bufflenArg.units = "bytes";
@@ -69,14 +71,24 @@ SoapySDR::ArgInfoList SoapyRTLSDR::getStreamArgsInfo(const int direction, const
 
     SoapySDR::ArgInfo buffersArg;
     buffersArg.key = "buffers";
-    buffersArg.value = "15";
-    buffersArg.name = "Buffer Count";
-    buffersArg.description = "Number of buffers per read.";
+    buffersArg.value = std::to_string(DEFAULT_NUM_BUFFERS);
+    buffersArg.name = "Ring buffers";
+    buffersArg.description = "Number of buffers in the ring.";
     buffersArg.units = "buffers";
     buffersArg.type = SoapySDR::ArgInfo::INT;
 
     streamArgs.push_back(buffersArg);
 
+    SoapySDR::ArgInfo asyncbuffsArg;
+    asyncbuffsArg.key = "asyncBuffs";
+    asyncbuffsArg.value = "0";
+    asyncbuffsArg.name = "Async buffers";
+    asyncbuffsArg.description = "Number of async usb buffers (advanced).";
+    asyncbuffsArg.units = "buffers";
+    asyncbuffsArg.type = SoapySDR::ArgInfo::INT;
+
+    streamArgs.push_back(asyncbuffsArg);
+
     return streamArgs;
 }
 
@@ -94,14 +106,12 @@ static void _rx_callback(unsigned char *buf, uint32_t len, void *ctx)
 void SoapyRTLSDR::rx_async_operation(void)
 {
     //printf("rx_async_operation\n");
-    rtlsdr_read_async(dev, &_rx_callback, this, numBuffers, bufferLength);
+    rtlsdr_read_async(dev, &_rx_callback, this, asyncBuffs, bufferLength);
     //printf("rx_async_operation done!\n");
 }
 
 void SoapyRTLSDR::rx_callback(unsigned char *buf, uint32_t len)
 {
-    std::unique_lock<std::mutex> lock(_buf_mutex);
-
     //printf("_rx_callback %d _buf_head=%d, numBuffers=%d\n", len, _buf_head, _buf_tail);
 
     //overflow condition: the caller is not reading fast enough
@@ -118,8 +128,15 @@ void SoapyRTLSDR::rx_callback(unsigned char *buf, uint32_t len)
 
     //increment the tail pointer
     _buf_tail = (_buf_tail + 1) % numBuffers;
+
+    //increment buffers available under lock
+    //to avoid race in acquireReadBuffer wait
+    {
+    std::lock_guard<std::mutex> lock(_buf_mutex);
     _buf_count++;
 
+    }
+
     //notify readStream()
     _buf_cond.notify_one();
 }
@@ -146,17 +163,17 @@ SoapySDR::Stream *SoapyRTLSDR::setupStream(
     }
 
     //check the format
-    if (format == "CF32")
+    if (format == SOAPY_SDR_CF32)
     {
         SoapySDR_log(SOAPY_SDR_INFO, "Using format CF32.");
         rxFormat = RTL_RX_FORMAT_FLOAT32;
     }
-    else if (format == "CS16")
+    else if (format == SOAPY_SDR_CS16)
     {
         SoapySDR_log(SOAPY_SDR_INFO, "Using format CS16.");
         rxFormat = RTL_RX_FORMAT_INT16;
     }
-    else if (format == "CS8") {
+    else if (format == SOAPY_SDR_CS8) {
         SoapySDR_log(SOAPY_SDR_INFO, "Using format CS8.");
         rxFormat = RTL_RX_FORMAT_INT8;
     }
@@ -203,12 +220,12 @@ SoapySDR::Stream *SoapyRTLSDR::setupStream(
         }
     }
 
-
-    if (args.count("buflen") != 0)
+    bufferLength = DEFAULT_BUFFER_LENGTH;
+    if (args.count("bufflen") != 0)
     {
         try
         {
-            int bufferLength_in = std::stoi(args.at("buflen"));
+            int bufferLength_in = std::stoi(args.at("bufflen"));
             if (bufferLength_in > 0)
             {
                 bufferLength = bufferLength_in;
@@ -218,6 +235,7 @@ SoapySDR::Stream *SoapyRTLSDR::setupStream(
     }
     SoapySDR_logf(SOAPY_SDR_DEBUG, "RTL-SDR Using buffer length %d", bufferLength);
 
+    numBuffers = DEFAULT_NUM_BUFFERS;
     if (args.count("buffers") != 0)
     {
         try
@@ -232,6 +250,19 @@ SoapySDR::Stream *SoapyRTLSDR::setupStream(
     }
     SoapySDR_logf(SOAPY_SDR_DEBUG, "RTL-SDR Using %d buffers", numBuffers);
 
+    asyncBuffs = 0;
+    if (args.count("asyncBuffs") != 0)
+    {
+        try
+        {
+            int asyncBuffs_in = std::stoi(args.at("asyncBuffs"));
+            if (asyncBuffs_in > 0)
+            {
+                asyncBuffs = asyncBuffs_in;
+            }
+        }
+        catch (const std::invalid_argument &){}
+    }
     if (tunerType == RTLSDR_TUNER_E4000) {
         IFGain[0] = 6;
         IFGain[1] = 9;
@@ -308,6 +339,13 @@ int SoapyRTLSDR::readStream(
         long long &timeNs,
         const long timeoutUs)
 {
+    //drop remainder buffer on reset
+    if (resetBuffer and bufferedElems != 0)
+    {
+        bufferedElems = 0;
+        this->releaseReadBuffer(stream, _currentHandle);
+    }
+
     //this is the user's buffer for channel 0
     void *buff0 = buffs[0];
 
@@ -422,15 +460,12 @@ int SoapyRTLSDR::acquireReadBuffer(
     long long &timeNs,
     const long timeoutUs)
 {
-    std::unique_lock <std::mutex> lock(_buf_mutex);
-
     //reset is issued by various settings
     //to drain old data out of the queue
     if (resetBuffer)
     {
         //drain all buffers from the fifo
-        _buf_head = (_buf_head + _buf_count) % numBuffers;
-        _buf_count = 0;
+        _buf_head = (_buf_head + _buf_count.exchange(0)) % numBuffers;
         resetBuffer = false;
         _overflowEvent = false;
     }
@@ -439,17 +474,17 @@ int SoapyRTLSDR::acquireReadBuffer(
     if (_overflowEvent)
     {
         //drain the old buffers from the fifo
-        _buf_head = (_buf_head + _buf_count) % numBuffers;
-        _buf_count = 0;
+        _buf_head = (_buf_head + _buf_count.exchange(0)) % numBuffers;
         _overflowEvent = false;
         SoapySDR::log(SOAPY_SDR_SSI, "O");
         return SOAPY_SDR_OVERFLOW;
     }
 
     //wait for a buffer to become available
-    while (_buf_count == 0)
+    if (_buf_count == 0)
     {
-        _buf_cond.wait_for(lock, std::chrono::microseconds(timeoutUs));
+        std::unique_lock <std::mutex> lock(_buf_mutex);
+        _buf_cond.wait_for(lock, std::chrono::microseconds(timeoutUs), [this]{return _buf_count != 0;});
         if (_buf_count == 0) return SOAPY_SDR_TIMEOUT;
     }
 
@@ -468,6 +503,5 @@ void SoapyRTLSDR::releaseReadBuffer(
     const size_t handle)
 {
     //TODO this wont handle out of order releases
-    std::unique_lock <std::mutex> lock(_buf_mutex);
     _buf_count--;
 }
